@@ -3,15 +3,13 @@
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
 use crate::blob_schema::BlobSchema;
-use crate::path_util;
 use alloc::rc::Rc;
-use alloc::string::String;
 use icu_provider::buf::BufferFormat;
 use icu_provider::prelude::*;
 use serde::de::Deserialize;
-use yoke::trait_hack::YokeTraitHack;
+use writeable::Writeable;
 use yoke::*;
-use zerovec::map::ZeroMapBorrowed;
+use zerovec::maps::{KeyError, ZeroMap2dBorrowed};
 
 /// A data provider loading data from blobs dynamically created at runtime.
 ///
@@ -23,36 +21,27 @@ use zerovec::map::ZeroMapBorrowed;
 /// # Examples
 ///
 /// ```
-/// use icu_locid_macros::langid;
+/// use icu_locid::locale;
 /// use icu_provider::prelude::*;
 /// use icu_provider::hello_world::*;
 /// use icu_provider_blob::BlobDataProvider;
-/// use std::fs::File;
-/// use std::io::Read;
-/// use std::rc::Rc;
+/// use std::fs;
 ///
 /// // Read an ICU4X data blob dynamically:
-/// let mut blob: Vec<u8> = Vec::new();
-/// let filename = concat!(
+/// let blob = fs::read(concat!(
 ///     env!("CARGO_MANIFEST_DIR"),
 ///     "/tests/data/hello_world.postcard",
-/// );
-/// File::open(filename)
-///     .expect("File should exist")
-///     .read_to_end(&mut blob)
-///     .expect("Reading pre-computed postcard buffer");
+/// )).expect("Reading pre-computed postcard buffer");
 ///
 /// // Create a DataProvider from it:
-/// let provider = BlobDataProvider::new_from_rc_blob(Rc::from(blob))
+/// let provider = BlobDataProvider::new_from_rc_blob(blob.into())
 ///     .expect("Deserialization should succeed");
 ///
 /// // Check that it works:
-/// let response: DataPayload<HelloWorldV1Marker> = provider.load_payload(
-///     &DataRequest {
-///         resource_path: ResourcePath {
-///             key: key::HELLO_WORLD_V1,
-///             options: langid!("la").into(),
-///         }
+/// let response: DataPayload<HelloWorldV1Marker> = provider
+///     .load_resource(&DataRequest {
+///         options: locale!("la").into(),
+///         metadata: Default::default(),
 ///     })
 ///     .expect("Data should be valid")
 ///     .take_payload()
@@ -63,7 +52,8 @@ use zerovec::map::ZeroMapBorrowed;
 ///
 /// [`StaticDataProvider`]: crate::StaticDataProvider
 pub struct BlobDataProvider {
-    data: Yoke<ZeroMapBorrowed<'static, str, [u8]>, Rc<[u8]>>,
+    #[allow(clippy::type_complexity)]
+    data: Yoke<ZeroMap2dBorrowed<'static, ResourceKeyHash, [u8], [u8]>, Rc<[u8]>>,
 }
 
 impl BlobDataProvider {
@@ -81,42 +71,39 @@ impl BlobDataProvider {
         })
     }
 
-    /// Gets the buffer for the given DataRequest out of the BlobSchema and returns it yoked
-    /// to the buffer backing the BlobSchema.
-    fn get_file(&self, req: &DataRequest) -> Result<Yoke<&'static [u8], Rc<[u8]>>, DataError> {
-        let path = path_util::resource_path_to_string(&req.resource_path);
-        // TODO: Distinguish between missing resource key and missing resource options
-        self.data
-            .try_project_cloned_with_capture::<&'static [u8], String, ()>(path, |zm, path, _| {
-                zm.get(&path).ok_or(())
-            })
-            .map_err(|_| DataErrorKind::MissingResourceKey.with_req(req))
-    }
-}
-
-impl<M> DataProvider<M> for BlobDataProvider
-where
-    M: DataMarker,
-    // Actual bound:
-    //     for<'de> <M::Yokeable as Yokeable<'de>>::Output: serde::de::Deserialize<'de>,
-    // Necessary workaround bound (see `yoke::trait_hack` docs):
-    for<'de> YokeTraitHack<<M::Yokeable as yoke::Yokeable<'de>>::Output>:
-        serde::de::Deserialize<'de>,
-{
-    fn load_payload(&self, req: &DataRequest) -> Result<DataResponse<M>, DataError> {
-        self.as_deserializing().load_payload(req)
+    #[cfg(feature = "export")]
+    #[doc(hidden)] // See #1771, we don't want this to be a publicly visible API
+    pub fn get_map(&self) -> &ZeroMap2dBorrowed<ResourceKeyHash, [u8], [u8]> {
+        self.data.get()
     }
 }
 
 impl BufferProvider for BlobDataProvider {
-    fn load_buffer(&self, req: &DataRequest) -> Result<DataResponse<BufferMarker>, DataError> {
-        let yoked_buffer = self.get_file(req)?;
+    fn load_buffer(
+        &self,
+        key: ResourceKey,
+        req: &DataRequest,
+    ) -> Result<DataResponse<BufferMarker>, DataError> {
         let mut metadata = DataResponseMetadata::default();
         // TODO(#1109): Set metadata.data_langid correctly.
         metadata.buffer_format = Some(BufferFormat::Postcard07);
         Ok(DataResponse {
             metadata,
-            payload: Some(DataPayload::from_yoked_buffer(yoked_buffer)),
+            payload: Some(DataPayload::from_yoked_buffer(
+                self.data
+                    .try_project_cloned_with_capture((key, req), |zm, (key, req), _| {
+                        zm.get(&key.get_hash(), req.options.write_to_string().as_bytes())
+                            .map_err(|e| {
+                                match e {
+                                    KeyError::K0 => DataErrorKind::MissingResourceKey,
+                                    KeyError::K1 => DataErrorKind::MissingResourceOptions,
+                                }
+                                .with_req(key, req)
+                            })
+                    })?,
+            )),
         })
     }
 }
+
+icu_provider::impl_auto_deserializing!(BlobDataProvider);
