@@ -2,18 +2,22 @@
 // called LICENSE at the top level of the ICU4X source tree
 // (online at: https://github.com/unicode-org/icu4x/blob/main/LICENSE ).
 
-use zerovec::map::ZeroMapBorrowed;
+use icu_provider::prelude::*;
+use zerovec::maps::ZeroMap2dBorrowed;
 
 /// A versioned Serde schema for ICU4X data blobs.
-#[derive(serde::Serialize, serde::Deserialize)]
+#[derive(serde::Deserialize)]
+#[cfg_attr(feature = "export", derive(serde::Serialize))]
 pub enum BlobSchema<'data> {
     #[serde(borrow)]
     V001(BlobSchemaV1<'data>),
 }
 
 /// Version 1 of the ICU4X data blob schema.
-#[derive(serde::Serialize, serde::Deserialize)]
+#[derive(serde::Deserialize)]
+#[cfg_attr(feature = "export", derive(serde::Serialize))]
 pub struct BlobSchemaV1<'data> {
+    /// Map from key hash and locale to buffer
     #[serde(borrow)]
-    pub resources: ZeroMapBorrowed<'data, str, [u8]>,
+    pub resources: ZeroMap2dBorrowed<'data, ResourceKeyHash, [u8], [u8]>,
 }
